@@ -25,6 +25,7 @@
 #include "list.h"
 #include "utils.h"
 #include "index.h"
+#include "xmalloc.h"
 
 /* List of words matching the current search. */
 /* """""""""""""""""""""""""""""""""""""""""" */
@@ -54,7 +55,7 @@ tst_insert(tst_node_t *p, wchar_t *w, void *data)
       p->eqkid = NULL;
     }
     else
-      p->eqkid = tst_insert(p->eqkid, ++w, data);
+      p->eqkid = tst_insert(p->eqkid, w + 1, data);
   }
   else
     p->hikid = tst_insert(p->hikid, w, data);
@@ -107,10 +108,25 @@ tst_traverse(tst_node_t *p, int (*callback)(void *), int first_call)
   return !!rc;
 }
 
+/* ======================================================== */
+/* Allocates and returns a newly created sub_tst_t element. */
+/* ======================================================== */
+sub_tst_t *
+sub_tst_new(void)
+{
+  sub_tst_t *elem = xmalloc(sizeof(sub_tst_t));
+
+  elem->size  = 64;
+  elem->count = 0;
+  elem->array = xmalloc(elem->size * sizeof(tst_node_t));
+
+  return elem;
+}
+
 /* ======================================================================= */
 /* Traverses the word tst looking for a wchar and build a list of pointers */
 /* containing all the sub-tst potentially leading to words containing the  */
-/* next wchar os the search string.                                        */
+/* next wchar of the search string.                                        */
 /* ======================================================================= */
 int
 tst_substring_traverse(tst_node_t *p,
@@ -233,11 +249,11 @@ tst_search(tst_node_t *root, wchar_t *w)
   return NULL;
 }
 
-/* ================================================================= */
-/* Searches all strings beginning with the same prefix.              */
-/* the callback function will be applied to each of theses strings   */
-/* returns NULL if no string matched the prefix.                     */
-/* ================================================================= */
+/* ============================================================== */
+/* Searches all strings beginning with the same prefix.           */
+/* the callback function will be applied to each of these strings */
+/* returns NULL if no string matched the prefix.                  */
+/* ============================================================== */
 void *
 tst_prefix_search(tst_node_t *root, wchar_t *w, int (*callback)(void *))
 {
@@ -268,58 +284,94 @@ tst_prefix_search(tst_node_t *root, wchar_t *w, int (*callback)(void *))
   return NULL;
 }
 
-/* ========================================================= */
-/* Insertion of an integer in a already sorted integer array */
-/* without duplications.                                     */
-/*                                                           */
-/* IN/OUT array : pointer to the array to manage.            */
-/* IN/OUT size  : allocated size of the array.               */
-/* IN/OUT nb    : number of longs in the array.              */
-/* IN value     : long value to be inserted in the array.    */
-/* ========================================================= */
-void
-insert_sorted_index(long **array_ptr, long *size, long *nb, long value)
+/* =============================================================== */
+/* Follows the eqkid list from a note in the tst until w if found. */
+/* Returns a pointer to the node if found, else returns NULL.      */
+/*                                                                 */
+/* Example with words "abcde", "bacdef" and "cda" in the tst_word  */
+/*         tree.                                                   */
+/*         if p points to the node (a), and w contains 'e' then    */
+/*         tst_search_in_word(p, w) will follow the starred nodes  */
+/*         until (e) and return q following the aqkid of the TST   */
+/*         from (a)                                                */
+/*                                                                 */
+/*    b                                                            */
+/*  / | \                                                          */
+/* a (a) c                                                         */
+/* |  |  |                                                         */
+/* b  c  d                                                         */
+/* |  |  |                                                         */
+/* c  d  a                                                         */
+/* |  |                                                            */
+/* d (e) <- q                                                      */
+/* |  |                                                            */
+/* e  f                                                            */
+/* =============================================================== */
+void *
+tst_search_in_word(tst_node_t *root, wchar_t w)
 {
-  long pos  = *nb;
-  long left = 0, right = *nb, middle;
+  tst_node_t *p  = root;
+  size_t      rc = 0;
 
-  if (*nb > 0)
+  while (p && p->splitchar != L'\0')
   {
-    /* Bisection search. */
-    /* """"""""""""""""" */
-    while (left < right)
+    if (p->splitchar == w)
     {
-      middle = (left + right) / 2;
-      if ((*array_ptr)[middle] == value)
-        return; /* Value already in array. */
-
-      if (value < (*array_ptr)[middle])
-        right = middle;
-      else
-        left = middle + 1;
+      rc = 1;
+      break;
     }
-    pos = left;
+    p = p->eqkid;
   }
 
-  if (*nb == *size)
+  if (rc)
+    return p;
+  else
+    return NULL;
+}
+
+/* ===================================================================== */
+/* Helper function used only in fuzzy mode to append a glyph in the      */
+/* search_list.                                                          */
+/* Takes all the level n pointers to nodes in tst_words to determine the */
+/* level n+1 pointers by following the eqkids pointers from these nodes. */
+/* ===================================================================== */
+void
+append_tst_search_list(char *glyph, long l)
+{
+  tst_node_t *tst_node, *node;
+  sub_tst_t  *sub_tst, *new_sub_tst;
+  wchar_t     w;
+
+  sub_tst = (sub_tst_t *)(tst_search_list->tail->data);
+  mbtowc(&w, glyph, l);
+
+  /* Create a new empty sub_tst_t node. */
+  /* """""""""""""""""""""""""""""""""" */
+  new_sub_tst = sub_tst_new();
+
+  /* For each eqkids of the level n */
+  /* """""""""""""""""""""""""""""" */
+  for (long i = 0; i < sub_tst->count; i++)
   {
-    *size += 64;
-    *array_ptr = xrealloc(*array_ptr, *size * sizeof(long));
+    tst_node = ((tst_node_t **)(sub_tst->array))[i];
+    if (tst_node->splitchar == L'\0')
+      continue;
+
+    /* If the search glyph exists as on of its eqkid. */
+    /* build a level n+1 sub_tst node.                */
+    /* """""""""""""""""""""""""""""""""""""""""""""" */
+    if ((node = tst_search_in_word(tst_node, w)) != NULL)
+    {
+      new_sub_tst->count++;
+      new_sub_tst->size  = new_sub_tst->count / 64 + 64;
+      new_sub_tst->array = xrealloc(new_sub_tst->array,
+                                    sizeof(tst_node_t *) * new_sub_tst->size);
+
+      new_sub_tst->array[new_sub_tst->count - 1] = node;
+    }
   }
 
-  /* Shift remaining array elements at position pos to position pos+1 */
-  /* (shift right one position).                                      */
-  /* """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""" */
-  if (*nb > pos)
-    memmove((*array_ptr) + pos + 1,
-            (*array_ptr) + pos,
-            sizeof(value) * (*nb - pos));
-
-  (*nb)++;
-
-  /* Set the value of the element at position pos in the passed array. */
-  /* """"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""" */
-  (*array_ptr)[pos] = value;
+  ll_append(tst_search_list, new_sub_tst);
 }
 
 /* ========================================================= */
