@@ -43,11 +43,14 @@ tst_fuzzy_traverse(tst_node_t *p,
                    int     first_call,
                    wchar_t w);
 
+void *
+tst_search_in_word(tst_node_t *root, wchar_t w);
+
 sub_tst_t *
 sub_tst_new(void);
 
 void
-insert_sorted_index(long **array, long *size, long *filled, long value);
+append_tst_search_list(char *glyph, long l);
 
 void
 insert_sorted_ptr(tst_node_t ***array,
@@ -59,10 +62,9 @@ insert_sorted_ptr(tst_node_t ***array,
 /* """""""""""""""""""""" */
 struct tst_node_s
 {
-  wchar_t splitchar;
-
   tst_node_t *lokid, *eqkid, *hikid;
   void       *data;
+  wchar_t     splitchar;
 };
 
 /* Structure to contain data and metadata attached to a fuzzy/substring. */
