@@ -13,11 +13,12 @@ int
 fputs_safe(const char * restrict s, FILE * restrict stream);
 
 int
-fputc_safe(int c, FILE * stream);
+fputc_safe(int c, FILE *stream);
 
 int
-tcsetattr_safe(int fildes, int optional_actions,
-               const struct termios * termios_p);
+tcsetattr_safe(int                   fildes,
+               int                   optional_actions,
+               const struct termios *termios_p);
 
 FILE *
 fopen_safe(const char * restrict stream, const char * restrict mode);
