@@ -42,22 +42,24 @@
 /* Typedefs. */
 /* ********* */
 
-typedef struct charsetinfo_s charsetinfo_t;
-typedef struct term_s        term_t;
-typedef struct toggle_s      toggle_t;
-typedef struct win_s         win_t;
-typedef struct word_s        word_t;
-typedef struct attrib_s      attrib_t;
-typedef struct limit_s       limit_t;
-typedef struct ticker_s      ticker_t;
-typedef struct misc_s        misc_t;
-typedef struct mouse_s       mouse_t;
-typedef struct sed_s         sed_t;
-typedef struct timeout_s     timeout_t;
-typedef struct output_s      output_t;
-typedef struct daccess_s     daccess_t;
-typedef struct search_data_s search_data_t;
-typedef struct attr_elem_s   attr_elem_t;
+typedef struct charsetinfo_s     charsetinfo_t;
+typedef struct term_s            term_t;
+typedef struct toggle_s          toggle_t;
+typedef struct win_s             win_t;
+typedef struct word_s            word_t;
+typedef struct attrib_s          attrib_t;
+typedef struct limit_s           limit_t;
+typedef struct ticker_s          ticker_t;
+typedef struct misc_s            misc_t;
+typedef struct mouse_s           mouse_t;
+typedef struct sed_s             sed_t;
+typedef struct timeout_s         timeout_t;
+typedef struct output_s          output_t;
+typedef struct daccess_s         daccess_t;
+typedef struct search_data_s     search_data_t;
+typedef struct attr_elem_s       attr_elem_t;
+typedef struct help_entry_s      help_entry_t;
+typedef struct help_attr_entry_s help_attr_entry_t;
 
 /* ****** */
 /* Enums. */
@@ -190,6 +192,9 @@ struct toggle_s
   int no_scrollbar;        /* 1 to disable the scrollbar display else 0.  */
   int no_hor_scrollbar;    /* 1 to disable the horizontab scrollbar       *
                             | display else 0.                             */
+  int hor_scrollbar;       /* 1 to always enable the scrollbar display    *
+                            * when lines are truncated in line/col mode   *
+                            * else 0.                                     */
   int blank_nonprintable;  /* 1 to try to display non-blanks in           *
                             | symbolic form else 0.                       */
   int keep_spaces;         /* 1 to keep the trailing spaces in columns    *
@@ -242,7 +247,7 @@ struct misc_s
   search_mode_t default_search_method;
   char          invalid_char_substitute;
   char          blank_char_substitute;
-  int           ignore_quotes;
+  char          ignore_quotes;
 };
 
 /* Structure to store mouse information. */
@@ -303,7 +308,7 @@ struct term_s
   char has_invis;             /* has invis mode.                         */
   char has_blink;             /* has blink mode.                         */
   char has_kmous;             /* has mouse reporting.                    */
-  char has_rep;               /* has repeact char.                       */
+  char has_rep;               /* has repeat char.                        */
 };
 
 /* Structure describing a word. */
@@ -313,22 +318,26 @@ struct word_s
   long           start, end;    /* start/end absolute horiz. word positions *
                                  | on the screen.                           */
   size_t         mb;            /* number of UTF-8 glyphs to display.       */
+  size_t         len_mb;        /* number of UTF-8 glyphs before filling    *
+                                 * the column.                              */
+  size_t         len;           /* number of bytes in str (for trimming).   */
+  int            offset;        /* may be > 0 in case of center or right    *
+                                 * alignment (# of spaces added).           */
   long           tag_order;     /* each time a word is tagged, this value.  *
                                  | is increased.                            */
   unsigned short tag_id;        /* tag id. 0 means no tag.                  */
+  unsigned char  is_matching;   /* word is matching a search ERE.           */
+  unsigned char  is_last;       /* 1 if the word is the last of a line.     */
+  unsigned char  is_selectable; /* word is selectable.                      */
+  unsigned char  is_numbered;   /* word has a direct access index.          */
   unsigned char  special_level; /* can vary from 0 to 9; 0 meaning normal.  */
   char          *str;           /* display string associated with this word */
-  size_t         len;           /* number of bytes of str (for trimming).   */
   char          *orig;          /* NULL or original string if is had been.  *
                                  | shortened for being displayed or altered *
                                  | by is expansion.                         */
   char          *bitmap;        /* used to store the position of the.       *
                                  | currently searched chars in a word. The  *
                                  | objective is to speed their display.     */
-  unsigned char  is_matching;   /* word is matching a search ERE.           */
-  unsigned char  is_last;       /* 1 if the word is the last of a line.     */
-  unsigned char  is_selectable; /* word is selectable.                      */
-  unsigned char  is_numbered;   /* word has a direct access index.          */
   attrib_t      *iattr;         /* Specific attribute set with the -Ra/-Ca  *
                                  | options.                                 */
 };
@@ -464,12 +473,12 @@ struct daccess_s
 /* """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""" */
 struct search_data_s
 {
-  char *buf;        /* search buffer.                            */
-  long  len;        /* current position in the search buffer.    */
-  long  utf8_len;   /* current position in the search buffer in  *
-                     | UTF-8 units.                              */
-  long *utf8_off_a; /* array of mb offsets in buf.               */
-  long *utf8_len_a; /* array of mb lengths in buf.               */
+  char *buf;    /* search buffer.                                */
+  long  len;    /* current position in the search buffer.        */
+  long  mb_len; /* current position in the search buffer in      *
+                 | UTF-8 units.                                  */
+  long *off_a;  /* array of mb offsets in buf.                   */
+  long *len_a;  /* array of mb lengths in buf.                   */
 
   int  err;           /* match error indicator.                  */
   long fuzzy_err_pos; /* last good position in search buffer.    */
@@ -490,12 +499,52 @@ struct attr_elem_s
   ll_t     *list; /* list of RE or intervals of columns. */
 };
 
+struct help_entry_s
+{
+  char *str; /* string to be displayed for an object in the help line. */
+  int   len; /* screen space taken by of one of these objects.         */
+  char *main_attr_str; /* Style attribute when colors are available.   */
+  char *alt_attr_str;  /* Style attribute when colors are missing.     */
+  char *flags;         /* string which can only contain occurrences of *
+                        | c: only on column/line mode,                 *
+                        | t: only in tag mode,                         *
+                        | m: -no_mouse is set.                         */
+};
+
+struct help_attr_entry_s
+{
+  char     *str;  /* string to be displayed for an object in the help line. */
+  int       len;  /* screen space taken by of one of these objects.         */
+  attrib_t *attr; /* attribute.                                             */
+};
+
 /* *********** */
 /* Prototypes. */
 /* *********** */
 
+attrib_t *
+attr_new(void);
+
+int
+help_add_entries(win_t               *win,
+                 term_t              *term,
+                 toggle_t            *toggles,
+                 help_attr_entry_t ***help_items_da,
+                 help_entry_t        *entries,
+                 int                  entries_nb);
+
+help_attr_entry_t ***
+init_help(win_t               *win,
+          term_t              *term,
+          toggle_t            *toggles,
+          long                 last_line,
+          help_attr_entry_t ***help_items_da);
+
 void
-help(win_t *win, term_t *term, long last_line);
+disp_help(win_t               *win,
+          term_t              *term,
+          help_attr_entry_t ***help_da,
+          int                  first_help_line);
 
 int
 tag_comp(void const *a, void const *b);
@@ -506,9 +555,15 @@ tag_swap(void **a, void **b);
 int
 isempty(const char *s);
 
+void
+clear_bitmap(word_t *word);
+
 void
 my_beep(toggle_t *toggles);
 
+void
+align_word(word_t *word, alignment_t alignment, size_t prerfix, char sp);
+
 int
 get_cursor_position(int * const r, int * const c);
 
@@ -819,12 +874,3 @@ get_clicked_index(win_t  *win,
                   int     line_click,
                   int     column_click,
                   int    *error);
-
-void
-align_word(word_t *word, alignment_t alignment, size_t prerfix, char sp);
-
-int
-isempty_non_utf8(const unsigned char *s);
-
-int
-isempty_utf8(const unsigned char *s);
