@@ -302,10 +302,10 @@ isprint8(int i)
 int
 my_wcscasecmp(const wchar_t *s1, const wchar_t *s2)
 {
-  wchar_t c1, c2;
-
   while (*s1)
   {
+    wchar_t c1, c2;
+
     c1 = towlower(*s1);
     c2 = towlower(*s2);
 
@@ -354,7 +354,10 @@ swap_string_parts(char **s, size_t first)
   size = strlen(*s);
 
   if (first > size)
+  {
+    free(tmp);
     return 0;
+  }
 
   strcpy(tmp, *s);
   strcat(tmp, *s);
@@ -476,15 +479,16 @@ hexdump(const char *buf, FILE *fp, const char *prefix, size_t size)
 int
 my_wcswidth(const wchar_t *s, size_t n)
 {
-  int len = 0;
-  int l   = 0;
-  int m   = 0;
+  int    len = 0;
+  size_t m   = 0;
 
   if (s == NULL || *s == L'\0')
     return 0;
 
   while (*s && m < n)
   {
+    int l;
+
     if ((l = wcwidth(*s)) >= 0)
     {
       /* Do not count zero-width-length glyphs. */
@@ -502,3 +506,219 @@ my_wcswidth(const wchar_t *s, size_t n)
 
   return len;
 }
+
+/* ==================================================================== */
+/* Get the target index of the number to be inserted in a sorted array. */
+/*                                                                      */
+/* IN array : an already empty or sorted array.                         */
+/* IN value : the value to be inserted in the array.                    */
+/* IN nb    : the number of existing elements in the array.             */
+/*                                                                      */
+/* RETURN   : -1 if the value is already in the array of the future     */
+/*            index of value in the array.                              */
+/* ==================================================================== */
+long
+get_sorted_array_target_pos(long *array, long nb, long value)
+{
+  long pos  = nb;
+  long left = 0, right = nb, middle;
+
+  if (nb > 0)
+  {
+    /* Bisection search. */
+    /* """"""""""""""""" */
+    while (left < right)
+    {
+      middle = (left + right) / 2;
+      if (array[middle] == value)
+        return -1; /* Value already in array. */
+
+      if (value < array[middle])
+        right = middle;
+      else
+        left = middle + 1;
+    }
+    pos = left;
+  }
+  return pos;
+}
+
+/* =============================================================== */
+/* Detect if the current terminal belongs to the foreground group. */
+/* returns 1 if yes else returns 0.                                */
+/* =============================================================== */
+int
+is_in_foreground_process_group(void)
+{
+  int fd, fg;
+
+  fd = open("/dev/tty", O_RDONLY);
+  if (fd < 0)
+    return 0;
+
+  fg = (tcgetpgrp(fd) == getpgid(0));
+
+  close(fd);
+
+  return fg;
+}
+
+/* ====================================================== */
+/* Returns 1 if a string is empty or only made of spaces. */
+/* Non UTF-8 version.                                     */
+/* ====================================================== */
+int
+isempty_non_utf8(const unsigned char *s)
+{
+  while (*s != '\0')
+  {
+    if (*s != ' ' && *s != '\t')
+      return 0;
+    s++;
+  }
+  return 1;
+}
+
+/* ====================================================== */
+/* Returns 1 if a string is empty or only made of spaces. */
+/* UTF-8 version.                                         */
+/* ====================================================== */
+int
+isempty_utf8(const unsigned char *s)
+{
+  unsigned char c, d;
+
+  while (*s != '\0')
+  {
+    if (*s == ' ' || *s == '\t') /* Normal ASCII spaces. */
+      goto next;
+
+    if (*s < 0xc2) /* Not an UTF-8 space -> return FALSE. */
+      return 0;
+
+    /* Scanning for a potential non UTF-8 spaces scanning. */
+    /* """"""""""""""""""""""""""""""""""""""""""""""""""" */
+    if ((c = *(s + 1)) != '\0')
+    {
+      if (*s == 0xc2 && (c == 0x85 || c == 0xa0))
+      {
+        s++;
+        goto next; /* Unnamed control character or NO-BREAK SPACE. */
+      }
+
+      if ((d = *(s + 2)) == '\0')
+        return 0;
+
+      if (*s == 0xe1 && c == 0x9a && d == 0x80)
+      {
+        s += 2;
+        goto next; /* OGHAM SPACE MARK. */
+      }
+
+      if (*s == 0xe1 && c == 0xa0 && d == 0x8e)
+      {
+        s += 2;
+        goto next; /* MONGOLIAN VOWEL SEPARATOR. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x80)
+      {
+        s += 2;
+        goto next; /* EN QUAD. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x81)
+      {
+        s += 2;
+        goto next; /* EM QUAD. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x82)
+      {
+        s += 2;
+        goto next; /* EN SPACE. */
+      }
+      if (*s == 0xe2 && c == 0x80 && d == 0x83)
+      {
+        s += 2;
+        goto next; /* EM SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x84)
+      {
+        s += 2;
+        goto next; /* THREE-PER-EM SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x85)
+      {
+        s += 2;
+        goto next; /* FOUR-PER-EM SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x86)
+      {
+        s += 2;
+        goto next; /* SIX-PER-EM SPACE. */
+      }
+      if (*s == 0xe2 && c == 0x80 && d == 0x87)
+      {
+        s += 2;
+        goto next; /* FIGURE SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x88)
+      {
+        s += 2;
+        goto next; /* PUNCTUATION SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x89)
+      {
+        s += 2;
+        goto next; /* THIN SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0x8a)
+      {
+        s += 2;
+        goto next; /* HAIR SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0xa8)
+      {
+        s += 2;
+        goto next; /* LINE SEPARATOR. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0xa9)
+      {
+        s += 2;
+        goto next; /* PARAGRAPH SEPARATOR. */
+      }
+
+      if (*s == 0xe2 && c == 0x80 && d == 0xaf)
+      {
+        s += 2;
+        goto next; /* NARROW NO-BREAK SPACE. */
+      }
+
+      if (*s == 0xe2 && c == 0x81 && d == 0x9f)
+      {
+        s += 2;
+        goto next; /* MEDIUM MATHEMATICAL SPACE. */
+      }
+
+      if (*s == 0xe3 && c == 0x80 && d == 0x80)
+      {
+        s += 2;
+        goto next; /* IDEOGRAPHIC SPACE. */
+      }
+
+      return 0;
+    }
+  next:
+    s++;
+  }
+  return 1;
+}
